# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Continuum Analytics, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
from os.path import dirname, join

import vcr

FIXTURES = join(dirname(__file__), 'fixtures')
VCR = vcr.VCR(cassette_library_dir=join(FIXTURES, 'cassettes'))
