



pip check
IF %ERRORLEVEL% NEQ 0 exit /B 1
anaconda-enterprise-cli --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
anaconda-enterprise-cli --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
aecli --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
aecli --version
IF %ERRORLEVEL% NEQ 0 exit /B 1
curl -OL https://curl.se/ca/cacert.pem
IF %ERRORLEVEL% NEQ 0 exit /B 1
set CA_CHAIN=%cd%\cacert.pem
IF %ERRORLEVEL% NEQ 0 exit /B 1
py.test tests
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
